# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-01-17 07:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djeddit', '0004_auto_20171212_1229'),
    ]

    operations = [
        migrations.AddField(
            model_name='thread',
            name='is_stickied',
            field=models.BooleanField(default=False),
        ),
    ]
